
#include <cmath>
#include "bfgs.h"
#include "shared.h"
#include "analysisFunction.h"
#include "analysisMaf.h"


typedef struct{
  int major;
  double *loglikes;
  int numInds;
  double *indF;
}bfgs_vars;




void frequency::printArg(FILE *argFile){
  fprintf(argFile,"%s:\n",__FILE__);
  fprintf(argFile,"-doMaf\t%d\n",doMaf);
  fprintf(argFile,"\t->1: BFGS frequency (known major minor)\n");
  fprintf(argFile,"\t->2: EM frequency (known major minor)\n");
  fprintf(argFile,"\t->4: BFGS frequency (unknown major minor)\n");
  fprintf(argFile,"\t->8: EM frequency (unknown major minor)\n");
  fprintf(argFile,"-doSNP\t%d\n",doSNP);
  fprintf(argFile,"-minInd\t%d\n",minInd);
  fprintf(argFile,"-minLRT\t%f\n",minLRT);
  fprintf(argFile,"-ref\t%s\n",refName);
  fprintf(argFile,"-anc\t%s\n",ancName);
  fprintf(argFile,"-doZ\t%d\n",doZ);
  fprintf(argFile,"\n");

}


//fancy little function
int isPowerOfTwo (unsigned int x)
{
  return ((x != 0) && ((x & (~x + 1)) == x));
}



double *frequency::indF = NULL;
void frequency::getOptions(argStruct *arguments){
  doZ =0;
  defualtMinMaf = -1;
  defaultMinLRT = -1;
  //default
  doMaf=0;
  GL=0;
  doSNP=0;
  doPost=0;
  minMaf=defualtMinMaf;
  minLRT=defaultMinLRT;
  //  minMaf=0;
  //  minLRT=-1;//this is everything
  minInd=0;
  emIter=100;
  doMajorMinor=0;
  refName = NULL;
  ancName = NULL;
  doMaf=angsd::getArg("-doMaf",doMaf,arguments);
  if(doMaf==-999){
    doMaf=0;
    printArg(stdout);
    exit(0);
  }

  if(doMaf==0)
    return;
  //  fprintf(stderr,"-domaf=%d\n",doMaf);
  GL=angsd::getArg("-GL",GL,arguments);
  doSNP=angsd::getArg("-doSNP",doSNP,arguments);
  doMajorMinor=angsd::getArg("-doMajorMinor",doMajorMinor,arguments);
  doPost=angsd::getArg("-doPost",doPost,arguments);
  minMaf=angsd::getArg("-minMaf",minMaf,arguments);
  minInd=angsd::getArg("-minInd",minInd,arguments);
  minLRT=angsd::getArg("-minLRT",minLRT,arguments);
  refName = angsd::getArg("-ref",refName,arguments);
  ancName = angsd::getArg("-anc",ancName,arguments);
  char* indFName = NULL;
  indFName = angsd::getArg("-indF",indFName,arguments);
  if(indFName ==NULL){
    indF = new double [arguments->nInd];
    for(int i=0;i<arguments->nInd;i++) indF[i] =0;
  }else
    indF = angsd::readDouble(indFName,arguments->nInd);

  //  fprintf(stderr,"refNAme=%s\nancNames=%s\n",refName,ancName);
  //  fprintf(stderr,"minLRT=%f\n",minLRT);
  if(!isPowerOfTwo((unsigned int) doMaf)){
    fprintf(stderr,"\n[%s] You have selected filters for maf/lrt\n",__FILE__);
    fprintf(stderr,"If you have selected more than one MAF estimator we will choose in following order\n");
    fprintf(stderr,"\t1. knownminor bfgs\n");
    fprintf(stderr,"\t2. knownminor EM\n");
    fprintf(stderr,"\t3. unknownminor bfgs\n");
    fprintf(stderr,"\t4. unknownminor EM\n");
    fprintf(stderr,"\t5. Posterior maf\n\n");
  }
  if(doSNP&&doMaf==0){
    fprintf(stderr,"You've selected snpcalling but no maf estimator please select -doMaf INT");
    exit(0);
  }
  if(minLRT!=defaultMinLRT &&(doSNP==0||doMaf==0)){
    fprintf(stderr,"You've selected minLRT threshold please also select -doMaf INT and doSNP INT");
    exit(0);
  }
  if((minMaf!=defualtMinMaf || minLRT!=defaultMinLRT)&&doMaf==0){
    fprintf(stderr,"\nYou've selected minmaf/minlrt and no MAF estimator, choose -doMaf\n\n");
    exit(0);
  }

  printArg(arguments->argumentFile);
}

//constructor
frequency::frequency(const char *outfiles,argStruct *arguments,int inputtype){
  //fprintf(stderr,"%s\n",__FUNCTION__);
  //tsk set start for EM
  EM_START = 0.001;

  
  getOptions(arguments);

  if(doMaf==0)
    return;
  /*
    inputtype 
    0) soap
    1) samglf 
    2) samglfClean 
    3) tglf 
    4) sim type 
    5) beagle
    6) mpileup
    7) bam
  */
  if(inputtype!=5&&inputtype!=0&&doMajorMinor==0){
    fprintf(stderr,"Error: you need to specify doMajorMinor in order to doMaf\n");
    exit(0);
  }

  if(inputtype==5&&doMaf!=16){
    fprintf(stderr,"Error: Only doMaf=16 can be performed on posterior input\n");
    exit(0);
  }
  if((inputtype==0 || inputtype==6 || inputtype==7)&&GL==0){
    fprintf(stderr,"Error: For sequence data (bam,SOAP) likehoods (-GL) must be specified for frequency estimation\n");
    exit(0);
  }
  if(inputtype!=5&&doMaf==16){
    fprintf(stderr,"Error: doMaf=16 can be performed on posterior input\n");
    exit(0);
  }
  if(doSNP&&doMaf==16){
    fprintf(stderr,"Error: doMaf=16 cannot be used for a likelihood ratio test (doSNP) \n");
    exit(0);
  }
  if(inputtype==5&&doPost){
    fprintf(stderr,"Error: Cannot estimate post (doPost) based on posterior probabilites\n");
    exit(0);
  }

  // fprintf(stderr,"Estimating major minor: %s\n",outfiles);
  //make output files
  const char* postfix;
  postfix=".mafs";
  if(doZ==1){
    postfix=".mafs.gz";
    outfileZ = openFileGz(outfiles,postfix,"wb");
  }else{
    postfix=".mafs";
    outfile = openFile(outfiles,postfix);
  }
  //print header
  kstring_t bufstr;
  bufstr.s=NULL;bufstr.l=bufstr.m=0;
  kputs("chromo\tposition\tmajor\tminor\t",&bufstr);
  if(refName!=NULL||inputtype==3)// inputtyp=3 is tglf, if tglf then -posi has been supplied
    kputs("ref\t",&bufstr);
  if(ancName||inputtype==3||inputtype==4)// inputtyp=3 is tglf, if tglf then -posi has been supplied
    kputs("ref\t",&bufstr);
  
  if(doMaf &1)
    kputs("knownBFGS\t",&bufstr);
  if(doMaf &2)
    kputs("knownEM\t",&bufstr);
  if(doMaf &4)
    kputs("unknownBFGS\t",&bufstr);
  if(doMaf &8)
    kputs("unknownEM\t",&bufstr);
  if(doMaf &16)
    kputs("PPmaf\t",&bufstr);
  
  if(doSNP){
    if(doMaf &1)
      kputs("pK-BFGS\t",&bufstr);
    if(doMaf &2)
      kputs("pK-EM\t",&bufstr);
    if(doMaf &4)
      kputs("pU-BFGS\t",&bufstr);
    if(doMaf &8)
      kputs("pu-EM\t",&bufstr);
  }
  kputs("nInd\n",&bufstr);
  if(doZ==1)
    gzwrite(outfileZ,bufstr.s,bufstr.l);
  else{
    fprintf(outfile,"%s",bufstr.s);
  }
}

//destructor
frequency::~frequency(){
  //  fprintf(stderr,"[%s] calling destructor\n",__FUNCTION__);
  //  fflush(stderr);
  if(doMaf==0)
    return;

  if(doZ==1)
    gzclose(outfileZ);
  else
    fclose(outfile);

  delete [] indF;
}



void frequency::prepPrint(funkyPars *pars){
  kstring_t *bufstr = new kstring_t ;
  bufstr->s=NULL; bufstr->l=bufstr->m=0;


  for(int s=0;s<pars->numSites;s++) {
    if(pars->keepSites[s]==0)
      continue;
    //plugin chr,pos,major,minor
    kputs(header->name[pars->refId],bufstr);kputc('\t',bufstr);
    kputw(pars->posi[s]+1,bufstr);kputc('\t',bufstr);
    kputc(intToRef[pars->major[s]],bufstr);kputc('\t',bufstr);
    kputc(intToRef[pars->minor[s]],bufstr);kputc('\t',bufstr);


    //plugin ref, anc if exists
    if(pars->ref!=NULL)
      {kputc(intToRef[pars->ref[s]],bufstr);kputc('\t',bufstr);}
    if(pars->anc!=NULL)
      {kputc(intToRef[pars->anc[s]],bufstr);kputc('\t',bufstr);}

    
    
    if(doMaf &1)
      ksprintf(bufstr,"%f\t",pars->results->freq->pml[s]);
    if(doMaf &2)
      ksprintf(bufstr,"%f\t",pars->results->freq->pEM[s]);
    if(doMaf &4)
      ksprintf(bufstr,"%f\t",pars->results->freq->pmlun[s]);
    if(doMaf &8)
      ksprintf(bufstr,"%f\t",pars->results->freq->pEMun[s]);
    if(doMaf &16)
      ksprintf(bufstr,"%f\t",pars->results->asso->freq[s]);


    if(doSNP){
      if(doMaf &1)
	ksprintf(bufstr,"%f\t",pars->results->freq->pmlSNP[s]);
      if(doMaf &2)
	ksprintf(bufstr,"%f\t",pars->results->freq->pEMSNP[s]);
      if(doMaf &4)
	ksprintf(bufstr,"%f\t",pars->results->freq->pmlunSNP[s]);
      if(doMaf &8)
	ksprintf(bufstr,"%f\t",pars->results->freq->pEMunSNP[s]);
    }


    if(pars->results->freq->keepInd[s]>pars->nInd){
      fprintf(stderr,"Possible corruption, samples with Information exceeds number of samples\n");
      fprintf(stderr,"[%s]site=%d\t%d\n",__FUNCTION__,s,pars->results->freq->keepInd[s]);
      exit(0);
    }
    else
      kputw(pars->results->freq->keepInd[s],bufstr);kputc('\n',bufstr);
  }

  pars->extras[index] = bufstr;
}




void frequency::print(funkyPars *pars) {
  if(doMaf==0)
    return;
  kstring_t *bufstr = (kstring_t *)pars->extras[index];
  if(doZ==1)
    gzwrite(outfileZ,bufstr->s,bufstr->l);  
  else if(bufstr->s!=0)
    fprintf(outfile,"%s",bufstr->s);
  free(bufstr->s);
  delete bufstr;

}



void frequency::clean(funkyPars *pars) {
  if(doMaf==0)
    return;


  //cleaning
  delete [] pars->results->asso->freq;
  delete [] pars->results->asso->lrt_snp;//maybe in doSNP
  delete [] pars->results->freq->pml;
  delete [] pars->results->freq->pmlun;
  delete [] pars->results->freq->pEM;
  delete [] pars->results->freq->pEMun;
  if(doSNP){
    delete [] pars->results->freq->pmlSNP;
    delete [] pars->results->freq->pmlunSNP;
    delete [] pars->results->freq->pEMSNP;
    delete [] pars->results->freq->pEMunSNP;
  }

  fflush(outfile);
}


void frequency::run(funkyPars *pars){
  if(doMaf==0)
    return;
  if(doMaf!=0){


    if(pars->post!=NULL)
      postFreq(pars);
    else if(pars->likes!=NULL){
      likeFreq(pars);
    } else{
      fprintf(stderr,"[%s] something is missing\n",__FUNCTION__);
    }
    
    if(pars->results->asso->freq==NULL){
      fprintf(stderr,"%s:Frequency not initilized\n",__FUNCTION__);
      exit(1);
    }
    
    for(int s=0;s<pars->numSites;s++){
      if(pars->keepSites[s]==0)
	continue;
      if(pars->results->asso->freq[s] < minMaf)
	pars->keepSites[s]=0;
      else if(pars->results->asso->freq[s] > 1 - minMaf)
	pars->keepSites[s]=0;
      else if(pars->results->freq->keepInd[s] < minInd)
	pars->keepSites[s]=0;
      else if(doSNP&&(pars->results->asso->lrt_snp[s] < minLRT))
	pars->keepSites[s]=0;
    }
  }
  if(doPost){
    if(pars->likes==NULL){
      fprintf(stderr,"[%s] likelihoods missing\n",__FUNCTION__);
      exit(0);
    }
    double **post = new double*[pars->numSites];
    double **like=angsd::get3likes(pars);
    for(int s=0;s<pars->numSites;s++){
      post[s] = new double [3*pars->nInd];

      if(pars->keepSites[s]==0)
	continue;
      if(doPost==1){//maf prior
	double freq=pars->results->asso->freq[s];
      for(int i=0;i<pars->nInd;i++){

	post[s][i*3+0]=like[s][i*3+0] + log(pow(1-freq,2) + freq*(1-freq)*indF[i]);
	post[s][i*3+1]=like[s][i*3+1] + log(2*freq*(1-freq) - 2*freq*(1-freq)*indF[i]);
	post[s][i*3+2]=like[s][i*3+2] + log(pow(freq,2) + freq*(1-freq)*indF[i]);
	double norm = angsd::addProtect3(post[s][i*3+0],post[s][i*3+1],post[s][i*3+2]);
	post[s][i*3+0]=exp(post[s][i*3+0]-norm);
	post[s][i*3+1]=exp(post[s][i*3+1]-norm);
	post[s][i*3+2]=exp(post[s][i*3+2]-norm);
	}
      }
      else if(doPost==2){//uniform prior
	for(int i=0;i<pars->nInd;i++){
	  double norm = angsd::addProtect3(like[s][i*3+0],like[s][i*3+1],like[s][i*3+2]);
	  for(int g=0;g<3;g++)
	    post[s][i*3+g]=exp(like[s][i*3+g]-norm);
	}
      }
      else{
	fprintf(stderr,"[%s] doPost must be 1 or 2 \n",__FUNCTION__);
	exit(0);
      
      }
      delete [] like[s];
    }
    pars->post = post; 
    delete[] like;

  }
  prepPrint(pars);
}


/*
Estimate the allele frequency from the posterior probabilities and add them to funkyPars
*/
void frequency::postFreq(funkyPars  *pars){
  double *returnFreq = new double[pars->numSites]; 

    for(int s=0;s<pars->numSites;s++){
      returnFreq[s]=0;
      for(int i=0;i<pars->nInd;i++){
	returnFreq[s]+=pars->post[s][i*3+1]+2*pars->post[s][i*3+2];
      }
    returnFreq[s] = returnFreq[s]/(pars->nInd*2);
    //fprintf(stderr,"returnfreq %f\n",returnFreq[s]);
    }
    pars->results->asso->freq=returnFreq;
}


void frequency::likeFreq(funkyPars *pars){//method=1: bfgs_known ;method=2 em;method=4 bfgs_unknown

  //here only the likelihoods for the three genotypes are used. 
  double **loglike = angsd::get3likes(pars);
 

  //the pml frequencies 
  double *pml=NULL;
  double *pEM =NULL;
  double *pmlun =NULL;
  double *pEMun =NULL;
  double *pmlSNP=NULL;
  double *pEMSNP =NULL;
  double *pmlunSNP =NULL;
  double *pEMunSNP =NULL;
  double *returnFreq = new double[pars->numSites]; //return value not to be deleted
  double *lrt_snp = new double[pars->numSites]; //return value not to be deleted
  
  //  fprintf(stderr,"method:=%d\n",method);
  if(doMaf &1){ 
    pml = new double[pars->numSites]; // numeric optimisation
    if(doSNP)
      pmlSNP = new double[pars->numSites]; // numeric optimisation
  }if(doMaf &2){
    pEM =new double[pars->numSites]; //em algorithm
    for(int i=0;i<pars->numSites;i++)
      pEM[i] = 0.0;
    if(doSNP)
      pEMSNP =new double[pars->numSites]; //em algorithm
  }if(doMaf &4){
    pmlun = new double[pars->numSites]; // numeric optimisation
    if(doSNP)
      pmlunSNP = new double[pars->numSites]; // numeric optimisation
  }if(doMaf &8){
    pEMun = new double[pars->numSites]; // EM optimisation
    if(doSNP)
      pEMunSNP = new double[pars->numSites]; // EM optimisation
  } 

  // number of individuals with data
  int *keepInd = pars->results->freq->keepInd;
  int keepList[pars->nInd];  

  //loop though all sites and check if we have data.

  for(int s=0;s<pars->numSites;s++) {
    if(keepInd[s]==0)//if we dont have any information
      continue;
    if(pars->keepSites[s]==0)//if we dont have any information
      continue;
    keepInd[s]=0;//
    for(int i=0 ; i<pars->nInd ;i++) {
      keepList[i]=1;
      if(loglike[s][i*3+0]+loglike[s][i*3+1]+loglike[s][i*3+2]>-0.0001){//anders{ //this also doesnt make sense we should loop trhough all 10 not 3.
	//	fprintf(stderr,"setting to zero\n");
	keepList[i]=0;
      }
      else{
	float max=angsd::getMax(loglike[s][i*3+0],loglike[s][i*3+1],loglike[s][i*3+2]);    
	if(max< -20)//THORFINN doesn't make sense when we are using unscaled likes         
	  keepList[i]=0;
	else
	  keepInd[s]++;
      }

    }


    if(doMaf &1) {

      pml[s]= likeFixedMinor_bfgs(loglike[s],pars->nInd);

      if(doSNP &1)
	pmlSNP[s] = 2*likeFixedMinor(0.0,loglike[s],pars->nInd)-2*likeFixedMinor(pml[s],loglike[s],pars->nInd);
    }

    if( doMaf &2) {

      if(pars->phat!=NULL)
	pEM[s]=emFrequency(loglike[s],pars->nInd,emIter,pars->phat[s],keepList,keepInd[s]);
      else
	pEM[s]=emFrequency(loglike[s],pars->nInd,emIter,EM_START,keepList,keepInd[s]);
      if(doSNP)
	pEMSNP[s] = 2*likeFixedMinor(0.0,loglike[s],pars->nInd)-2*likeFixedMinor(pEM[s],loglike[s],pars->nInd);
    }
    if( doMaf &4) {
      pmlun[s]= likeNoFixedMinor_bfgs(pars->likes[s],pars->nInd,pars->major[s]);
      if(doSNP)
	pmlunSNP[s]= 2*likeNoFixedMinor(0.0,pars->likes[s],pars->nInd,pars->major[s])-2*likeNoFixedMinor(pmlun[s],pars->likes[s],pars->nInd,pars->major[s]);
      
    }
    if( doMaf &8 ){
      if(pars->phat!=NULL)
	pEMun[s]=emFrequencyNoFixed(pars->likes[s],pars->nInd,emIter,pars->phat[s],keepList,keepInd[s],pars->major[s],pars->minor[s]);
      else
      	pEMun[s]=emFrequencyNoFixed(pars->likes[s],pars->nInd,emIter,EM_START,keepList,keepInd[s],pars->major[s],pars->minor[s]);
      if(doSNP)
	pEMunSNP[s] = 2*likeNoFixedMinor(0.0,pars->likes[s],pars->nInd,pars->major[s])-2*likeNoFixedMinor(pEMun[s],pars->likes[s],pars->nInd,pars->major[s]);

    }
  }

  pars->results->freq->pml=pml;
  pars->results->freq->pEM=pEM;
  pars->results->freq->pmlun=pmlun;
  pars->results->freq->pEMun=pEMun;
  pars->results->freq->pmlSNP=pmlSNP;
  pars->results->freq->pEMSNP=pEMSNP;
  pars->results->freq->pmlunSNP=pmlunSNP;
  pars->results->freq->pEMunSNP=pEMunSNP;



  //  pars->print->maf
 // fprintf(stderr,"knsfsdfsdfsfsdfow %f\n", pars->results->freq->pEM[0]);
  //  fprintf(stderr,"knsfsdfsdfsfsdfow %f s:%d\n",pars->print->freq->maf[1][4],4);


  for(int i=0;i<pars->numSites;i++)
    delete [] loglike[i];
  delete [] loglike;

  
  //anders: this is less stupid but slower
  //this is abit stupid, but in the case where multiple MAFS has been selected, we use the max(method)
  //thorfinn: this is very stupid
  for(int s=0;s<pars->numSites;s++){
    if(doMaf &8)
      returnFreq[s]=pEMun[s];
    else if(doMaf &4)
      returnFreq[s]=pmlun[s];
    else if(doMaf &2 )
      returnFreq[s]=pEM[s];
    else if(doMaf &1 )
      returnFreq[s]=pml[s];
  }
  //thorfinn april 16 2012
  for(int s=0;s<pars->numSites;s++){
    if((doMaf &8) && doSNP)
      lrt_snp[s]=pEMunSNP[s];
    else if((doMaf &4) && doSNP)
      lrt_snp[s]=pmlunSNP[s];
    else if((doMaf &2) && doSNP )
      lrt_snp[s]=pEMSNP[s];
    else if((doMaf &1) && doSNP)
      lrt_snp[s]=pmlSNP[s];
  }




  //clean up is done in the printout
  //return returnFreq;
  pars->results->asso->freq=returnFreq;
  
  pars->results->asso->lrt_snp = lrt_snp;//thorfinn add 16april 2012

}

double frequency::likeFixedMinor_bfgs(double *loglikes,int numInds){

  bfgs_vars *bv = new bfgs_vars;
  bv->loglikes = loglikes;
  bv->numInds = numInds;
  bv->indF = indF;
  double start[1] = {0.01};
  double lbound[1] = {0};
  double ubound[1] = {1};
  int lims[1] = {2};


  double res=findmax_bfgs(1,start,bv,&likeFixedMinor_wrapper,NULL,lbound,ubound,lims,-1);
  //fprintf(stderr,"%s\t like %f isoptim is: %f\n",__FUNCTION__,res,start[0]);	
  delete bv;
  return start[0];

}
///* unstaticing when possible
double frequency::likeFixedMinor_wrapper(const double *para,const void *dats){

  bfgs_vars *bf =(bfgs_vars *) dats;
  double *loglikes = bf->loglikes;
  int numInds = bf->numInds;
  //  double *indF = bf->indF;

  double res = likeFixedMinor(para[0],loglikes,numInds);

  return res;
}
//*/



double frequency::emFrequencyNoFixed(double *loglike,int numInds, int iter,double start,int *keep,int keepInd,int major,int minor){

  int iMinor[3];
  int n=0;
  for(int i=0;i<4;i++){
    if(i!=major){
      iMinor[n]=i;
      n++;
    }
  }
  
  
  double **loglikeGeno;
  loglikeGeno =new double*[3];
  
  for(int j=0;j<3;j++){
    loglikeGeno[j] = new double[numInds*3]; 
    for(int i=0;i<numInds;i++){
      loglikeGeno[j][i*3+0]=loglike[i*10+angsd::majorminor[major][major]];
      loglikeGeno[j][i*3+1]=loglike[i*10+angsd::majorminor[major][iMinor[j]]];
      loglikeGeno[j][i*3+2]=loglike[i*10+angsd::majorminor[iMinor[j]][iMinor[j]]];
    }
  }
  

  //float accu=0.00001;
  //float accu2=0.01;
  float W0[3];
  float W1[3];
  float W2[3];
  start=0.4;
  float p=(float)start;
  float temp_p=(float)start;
  double accu=0.00001;
  double accu2=0;
  float sum;
  
  int it=0;
  float weight[3];
  float normWeight[3];
  float norm;
  int correctInd;
  //float likeun;
  //float partialLike[3];

  for(it=0;it<iter;it++){
    //fprintf(stderr,"[%s].p=%f\n",__FUNCTION__,p);
    //likeun=likeNoFixedMinor(p,loglike,keepInd,major);
    for(int j=0;j<3;j++)
      weight[j]=-likeFixedMinor(p,loglikeGeno[j],keepInd);
    norm=angsd::addProtect3(weight[0],weight[1],weight[2]);
    for(int j=0;j<3;j++)
      normWeight[j]=exp(weight[j]-norm);
    sum=0;
    correctInd=keepInd;
    for(int i=0;i<numInds;i++){
      if(keep[i]==0)
        continue;
      for(int j=0;j<3;j++){
	W0[j]=exp(loglike[i*10+angsd::majorminor[major][major]]) * (pow(1-p,2) + (1-p)*p*indF[i]);
	W1[j]=exp(loglike[i*10+angsd::majorminor[major][iMinor[j]]]) * (2*(1-p)*p - 2*(1-p)*p*indF[i]);
	W2[j]=exp(loglike[i*10+angsd::majorminor[iMinor[j]][iMinor[j]]]) * (pow(p,2) + (1-p)*p*indF[i]);

	if(W0[j]+W1[j]+W2[j]<0.0000001)
	  continue;

	//	correctInd++;
	sum+=(W1[j]+2*W2[j])/(2*(W0[j]+W1[j]+W2[j]))*normWeight[j];
	
      }
      
    }
    //fprintf(stderr,"%f | %f %f %f | %f %f %f | %f | %d %d | %f | %d\n",p, normWeight[0], normWeight[1], normWeight[2],partialLike[0],partialLike[1],partialLike[2],addProtect3(weight[0],weight[1],weight[2]),major,minor,likeun,correctInd);
    p=sum/correctInd;

       
    if((p-temp_p<accu&&temp_p-p<accu)||(p/temp_p<1+accu2&&p/temp_p>1-accu2))
      break;
       
    temp_p=p;
    if(isnan(p)){
      fprintf(stderr,"[%s] caught nan will exit\n",__FUNCTION__);
      for(int j=0;j<3;j++)
	fprintf(stderr,"w%d %f ",j,weight[j]);
      fflush(stderr);

      for(int i=0;i<numInds;i++){
	for(int j=0;j<10;j++){
	  if(keep[i]==0)
	    continue;
	  fprintf(stderr,"%f\t",loglike[i*10+j]);
	}
	fprintf(stderr,"\n");
      }
      fprintf(stderr,"%f | %f %f %f |%f | %d %d | \n",p, normWeight[0], normWeight[1], normWeight[2],angsd::addProtect3(weight[0],weight[1],weight[2]),major,minor);
      
      //exit(0);
    }
  }
  //fprintf(stderr,"---------------------------------------\n");

  
 for(int j=0;j<3;j++)
   delete[] loglikeGeno[j]; 
 delete[] loglikeGeno;

 return(p);
}



float frequency::likeNoFixedMinor(float p,double *logLikes,int numInds,int major){
  //logLikes contains the 10 likelihoods for one site
  //bfgs must be used
  float partialLike[3];
  for(int j=0;j<3;j++)
    partialLike[j]=0;
  float totalLike=0;
  int iMinor[3];
  int n=0;
  for(int i=0;i<4;i++){
    if(i!=major){
      iMinor[n]=i;
      n++;
    }
  }

  for(int i=0;i<numInds;i++){
    for(int j=0;j<3;j++) 
      partialLike[j]+=angsd::addProtect3(
					 logLikes[i*10+angsd::majorminor[major][major]] + log(pow(1-p,2) + (1-p)*p*indF[i]),
					 logLikes[i*10+angsd::majorminor[major][iMinor[j]]] + log(2*(1-p)*p - 2*(1-p)*p*indF[i]),
					 logLikes[i*10+angsd::majorminor[iMinor[j]][iMinor[j]]] + log(pow(p,2) + (1-p)*p*indF[i]));
  }
  totalLike=angsd::addProtect3(partialLike[0],partialLike[1],partialLike[2]);
  return -totalLike;

}



double frequency::likeNoFixedMinor_wrapper(const double *para,const void *dats){
  bfgs_vars *bf = (bfgs_vars*) dats;

  return likeNoFixedMinor(para[0],bf->loglikes,bf->numInds,bf->major);
}

double frequency::likeNoFixedMinor_bfgs(double *loglikes,int numInds,int major){
  bfgs_vars bv;
  bv.loglikes = loglikes;
  bv.numInds = numInds;
  bv.major=major;
  bv.indF=indF;
  double start[1] = {0.01};
  double lbound[1] = {0};
  double ubound[1] = {1};
  int lims[1] = {2};
  double res=findmax_bfgs(1,start,(void*) &bv,likeNoFixedMinor_wrapper,NULL,lbound,ubound,lims,-1);
  //fprintf(stderr,"%f\n",res);
  //fprintf(stderr,"%s\t like %f isoptim is: %f\n",__FUNCTION__,res,start[0]);	
  return start[0];
}

double frequency::likeFixedMinor(double p,double *logLikes,int numInds){

  double totalLike=0;
  for(int i=0;i<numInds;i++){
    totalLike+=angsd::addProtect3(
				  logLikes[i*3+0] + log(pow(1-p,2) + (1-p)*p*indF[i]),
				  logLikes[i*3+1] + log(2*(1-p)*p - 2*(1-p)*p*indF[i]),
				  logLikes[i*3+2] + log(pow(p,2) + (1-p)*p*indF[i]));
  }

  return -totalLike;
}




double frequency::emFrequency(double *loglike,int numInds, int iter,double start,int *keep,int keepInd){

  if(keepInd == 0)
    return 0.0;
  
  float W0;
  float W1;
  float W2;
  // fprintf(stderr,"start=%f\n",start);
  float p=(float)start;
  float temp_p=(float)start;
  double accu=0.00001;
  double accu2=0;
  float sum;


  int it=0;
  
  for(it=0;it<iter;it++){
    sum=0;
    for(int i=0;i<numInds;i++){
      if(keep!=NULL && keep[i]==0)
        continue;

      W0=exp(loglike[i*3+0]) * (pow(1-p,2) + (1-p)*p*indF[i]);
      W1=exp(loglike[i*3+1]) * (2*(1-p)*p - 2*(1-p)*p*indF[i]);
      W2=exp(loglike[i*3+2]) * (pow(p,2) + (1-p)*p*indF[i]);

      sum+=(W1+2*W2)/(2*(W0+W1+W2));
      //  fprintf(stderr,"%f %f %f\n",W0,W1,W2);
      if(0&&isnan(sum)){
	//fprintf(stderr,"PRE[%d]: W %f\t%f\t%f sum=%f\n",i,W0,W1,W2,sum);
	exit(0);
      }
    }

    p=sum/keepInd;
    // fprintf(stderr,"it=%d\tp=%f\tsum=%f\tkeepInd=%d\n",it,p,log(sum),keepInd);
    if((p-temp_p<accu&&temp_p-p<accu)||(p/temp_p<1+accu2&&p/temp_p>1-accu2))
      break;
    temp_p=p;
  }


  
  //  fprintf(stderr,"THORFINNN iter %d\n",it);

  if(isnan(p)){
    fprintf(stderr,"[%s] caught nan will not exit\n",__FUNCTION__);
    fprintf(stderr,"logLike (3*nInd). nInd=%d\n",numInds);
    //print_array(stderr,loglike,3*numInds);
    fprintf(stderr,"keepList (nInd)\n");
    //print_array(stderr,keep,numInds);
    fprintf(stderr,"used logLike (3*length(keep))=%d\n",keepInd);

    for(int ii=0;0&&ii<numInds;ii++){
      if(keep!=NULL && keep[ii]==1)
	    fprintf(stderr,"1\t");
	for(int gg=0;gg<3;gg++)
	  fprintf(stderr,"%f\t",loglike[ii*3+gg]);
      fprintf(stderr,"\n");
    }
    sum=0;
    for(int i=0;i<numInds;i++){
      if(keep!=NULL && keep[i]==0)
        continue;

      W0=exp(loglike[i*3+0]) * (pow(1-p,2) + (1-p)*p*indF[i]);
      W1=exp(loglike[i*3+1]) * (2*(1-p)*p - 2*(1-p)*p*indF[i]);
      W2=exp(loglike[i*3+2]) * (pow(p,2) + (1-p)*p*indF[i]);

      sum+=(W1+2*W2)/(2*(W0+W1+W2));
      //fprintf(stderr,"p=%f W %f\t%f\t%f sum=%f loglike: %f\n",p,W0,W1,W2,sum,exp(loglike[i*3+2])*pow(1-p,2));
    }
    p=-999;
    // exit(0);
  }
  
  return(p);
}
